/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: basemode.c,v 1.4 1999/06/08 15:14:58 kato Exp $
 */

#include <Xm/Xm.h>

#include <stdlib.h>

#include "const.h"
#include "stack.h"
#include "keyboard.h"
#include "basemode.h"
#include "operator.h"
#include "option.h"

static void	clearstackall_base(void);
static void	printstack_base(char *v, int index);
static void	stackClearX_base(void);
static void	stackCopyY_base(void);
static void	stackCopyZ_base(void);
static void	stackCopyT_base(void);
static void	stackDown_base(void);
static void	stackEnter_base(char* xchar, int stackXpos);
static void*	stackGetX_base(char *xchar, int stackXpos);
static void	stackMem_base(char *xchar, int stackXpos);
static void	stackMemAdd_base(char *xchar, int stackXpos);
static void	stackMemRead_base(void);
static void	stackMemSub_base(char *xchar, int stackXpos);
static void	stackPush_base(void* d);
static void*	stackPop_base(char* xchar, int stackXpos);
static void	stackSetX_base(void* d);
static void	stackSign_base(char* xchar, int stackXpos);
static void	stackSwap_base(char* xchar, int stackXpos);
static void	stackUp_base(char* xchar, int stackXpos);
static void	stackupward_base(void);

static void	keyevent_base(XKeyEvent* ev);
static void	keypush_base(int keycode);
static void	setkeytop1_base(int code);

static struct stack_op_t comp_stack_op = {
	printstack_base,
	clearstackall_base,
	stackupward_base,
	stackEnter_base,
	stackPush_base,
	stackPop_base,
	stackGetX_base,
	stackSetX_base,
	stackSwap_base,
	stackUp_base,
	stackDown_base,
	stackSign_base,
	stackClearX_base,
	stackMem_base,
	stackMemRead_base,
	stackMemAdd_base,
	stackMemSub_base,
	stackCopyY_base,
	stackCopyZ_base,
	stackCopyT_base,
};

static struct keyboard_op_t comp_keyboard_op = {
	setkeytop1_base,
	keypush_base,
	keyevent_base,
	{
		"a", "b", "c", "d", "e", "f", "m",
		"G", "H", "I", "J", "K", "L", "M",
		"n", "o", "p", "q", "r", "s", "t",
		"ENTER",  "P", "Q", "R", "S", "T",
		"F1", "F2", "F3", "F4", "Tab", "Esc",
		"7", "8", "9", "BS", "Home", "Up",
		"4", "5", "6", "*", "/", "Down",
		"1", "2", "3", "+", "-", "F10",
		"0", "~", "z", "F5", "F6", "F7"
	}
};

static int	value_base[NSTACK];
static int	memory_base;
static int	basesubmode = BASE_DEC;

static char	*keytop[NKEY] = {
	"A", "B", "C", "D", "E", "F", "XNOR",
	"BIN", "OCT", "DEC", "HEX", "OR", "AND", "XOR",
	"Y%X", "X!", "log", "X++", "sqrt", "Y^X", "Y<<X",
	"ENTER", "10^X", "X--", "X^2", "cbrt", "Y>>X",
	"MEM", "MR", "M+", "M-", "SWAP", "OFF",
	"7", "8", "9", "BS", "CLR", "UP",
	"4", "5", "6", "*", "/", "DOWN",
	"1", "2", "3", "+", "-", "+/-",
	"0", "NOT", "NEG2", "Y", "Z", "T"
};

static void
keypush_base(int keycode)
{
	switch ((int)keycode) {
	case KEY_OFF:
		exit(0);
		/* NOTREACHED */
		break;
	case KEY_0:
		stackAddChar('0');
		break;
	case KEY_1:
		stackAddChar('1');
		break;
	case KEY_2:
		if (basesubmode != BASE_BIN)
			stackAddChar('2');
		break;
	case KEY_3:
		if (basesubmode != BASE_BIN)
			stackAddChar('3');
		break;
	case KEY_4:
		if (basesubmode != BASE_BIN)
			stackAddChar('4');
		break;
	case KEY_5:
		if (basesubmode != BASE_BIN)
			stackAddChar('5');
		break;
	case KEY_6:
		if (basesubmode != BASE_BIN)
			stackAddChar('6');
		break;
	case KEY_7:
		if (basesubmode != BASE_BIN)
			stackAddChar('7');
		break;
	case KEY_8:
		if (basesubmode == BASE_DEC || basesubmode == BASE_HEX)
			stackAddChar('8');
		break;
	case KEY_9:
		if (basesubmode == BASE_DEC || basesubmode == BASE_HEX)
			stackAddChar('9');
		break;
	case KEY_SIN:
		if (basesubmode == BASE_HEX)
			stackAddChar('a');
		break;
	case KEY_COS:
		if (basesubmode == BASE_HEX)
			stackAddChar('b');
		break;
	case KEY_TAN:
		if (basesubmode == BASE_HEX)
			stackAddChar('c');
		break;
	case KEY_SINH:
		if (basesubmode == BASE_HEX)
			stackAddChar('d');
		break;
	case KEY_COSH:
		if (basesubmode == BASE_HEX)
			stackAddChar('e');
		break;
	case KEY_TANH:
		if (basesubmode == BASE_HEX)
			stackAddChar('f');
		break;

	case KEY_ASIN:
		stackGetX();
		basesubmode = BASE_BIN;
		optionlabel_basemode();
		displayall();
		break;
	case KEY_ACOS:
		stackGetX();
		basesubmode = BASE_OCT;
		optionlabel_basemode();
		displayall();
		break;
	case KEY_ATAN:
		stackGetX();
		basesubmode = BASE_DEC;
		optionlabel_basemode();
		displayall();
		break;
	case KEY_ASINH:
		stackGetX();
		basesubmode = BASE_HEX;
		optionlabel_basemode();
		displayall();
		break;

	case KEY_POINT:
		op_not_int();
		break;
	case KEY_EXP:
		op_neg2_int();
		break;

	case KEY_ADD:
		if (isjustaftere)
			stackAddChar('+');
		else {
			op_add_int();
		}
		break;
	case KEY_SUB:
		if (isjustaftere)
			stackAddChar('-');
		else {
			op_sub_int();
		}
		break;
	case KEY_MUL:
		op_mul_int();
		break;
	case KEY_DIV:
		op_div_int();
		break;
	case KEY_POW:
		op_pow_int();
		break;
	case KEY_SHIFTR:
		op_shiftR_int();
		break;
	case KEY_SHIFTL:
		op_shiftL_int();
		break;

	case KEY_ACOSH:
		op_or_int();
		break;
	case KEY_ATANH:
		op_and_int();
		break;
	case KEY_E:
		op_xor_int();

		break;
	case KEY_LOG:
		op_log_int();
		break;
	case KEY_ANTILOG:
		op_antilog_int();
		break;
	case KEY_LN:
		op_inc_int();
		break;
	case KEY_ANTILN:
		op_dec_int();
		break;
	case KEY_SQRT:
		op_sqrt_int();
		break;
	case KEY_CBRT:
		op_cbrt_int();
		break;
	case KEY_SQUARE:
		op_square_int();
		break;
	case KEY_RECIP:
		op_mod_int();
		break;
	case KEY_FACTO:
		op_facto_int();
		break;

	case KEY_SWAP:
		stackSwap();
		break;
	case KEY_BS:
		stackBS();
		break;
	case KEY_ENTER:
		stackEnter();
		break;
	case KEY_UP:
		stackUp();
		break;
	case KEY_DOWN:
		stackDown();
		break;
	case KEY_SIGN:
		stackSign();
		break;
	case KEY_CLR:
		stackClear();
		break;
	case KEY_MEM:
		stackMem();
		break;
	case KEY_MR:
		stackMemRead();
		break;
	case KEY_MPLUS:
		stackMemAdd();
		break;
	case KEY_MSUB:
		stackMemSub();
		break;
	case KEY_Y:
		stackCopyY();
		break;
	case KEY_Z:
		stackCopyZ();
		break;
	case KEY_T:
		stackCopyT();
		break;
	case KEY_PI:
		op_xnor_int();
		break;
	}
}

static void
keyevent_base(XKeyEvent* ev)
{
	char	str[16], *keystring;
	KeySym	keysym;
	XComposeStatus	composestatus;
	int	len;

	len = XLookupString(ev, str, 15, &keysym, &composestatus);
	str[len] = '\0';

	keystring = XKeysymToString(keysym);

	switch (str[0]) {
	case 0x1b:	/* ESC */
		exit(0);
		/* NOTREACHED */
		break;
	case '0':
		stackAddChar(str[0]);
		break;
	case '1':
		stackAddChar(str[0]);
		break;
	case '2':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '3':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '4':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '5':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '6':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '7':
		if (basesubmode != BASE_BIN)
			stackAddChar(str[0]);
		break;
	case '8':
		if (basesubmode == BASE_DEC || basesubmode == BASE_HEX)
			stackAddChar(str[0]);
		break;
	case '9':
		if (basesubmode == BASE_DEC || basesubmode == BASE_HEX)
			stackAddChar(str[0]);
		break;
	case 'a':
	case 'A':
	case 'b':
	case 'B':
	case 'c':
	case 'C':
	case 'd':
	case 'D':
	case 'e':
	case 'E':
	case 'f':
	case 'F':
		if (basesubmode == BASE_HEX)
			stackAddChar(str[0]);
		break;
	case '+':
		op_add_int();
		break;
	case '-':
		op_sub_int();
		break;
	case '*':
		op_mul_int();
		break;
	case '/':
		op_div_int();
		break;
	case '^':
		op_pow_int();
		break;

	case '!':
		op_facto_int();
		break;
	case '~':
		op_not_int();
		break;
	case 'z':
		op_neg2_int();
		break;

	case 0x08:	/* BS */
		stackBS();
		break;
	case 0x0d:	/* Enter */
		stackEnter();
		break;

	case 'G':
		stackGetX();
		basesubmode = BASE_BIN;
		optionlabel_basemode();
		displayall();
		break;
	case 'H':
		stackGetX();
		basesubmode = BASE_OCT;
		optionlabel_basemode();
		displayall();
		break;
	case 'I':
		stackGetX();
		basesubmode = BASE_DEC;
		optionlabel_basemode();
		displayall();
		break;
	case 'J':
		stackGetX();
		basesubmode = BASE_HEX;
		optionlabel_basemode();
		displayall();
		break;

	case 'K':
		op_or_int();
		break;
	case 'L':
		op_and_int();
		break;
	case 'M':
		op_xor_int();
		break;

	case 'm':
		op_xnor_int();
		break;

	case 'n':
		op_mod_int();
		break;
	case 'o':
		op_facto_int();
		break;
	case 'p':
		op_log_int();
		break;
	case 'P':
		op_antilog_int();
		break;
	case 'q':
		op_inc_int();
		break;
	case 'Q':
		op_dec_int();
		break;
	case 'r':
		op_sqrt_int();
		break;
	case 'R':
		op_square_int();
		break;
	case 's':
		op_pow_int();
		break;
	case 'S':
		op_cbrt_int();
		break;
	case 't':
		op_shiftL_int();
		break;
	case 'T':
		op_shiftR_int();
		break;

	case 9:	/* TAB */
		stackSwap();
		break;

	default:
		if (strcmp(keystring, "Home") == 0)
			stackClear();
		else if (strcmp(keystring, "F1") == 0)
			stackMem();
		else if (strcmp(keystring, "F2") == 0)
			stackMemRead();
		else if (strcmp(keystring, "F3") == 0)
			stackMemAdd();
		else if (strcmp(keystring, "F4") == 0)
			stackMemSub();
		else if (strcmp(keystring, "F5") == 0)
			stackCopyY();
		else if (strcmp(keystring, "F6") == 0)
			stackCopyZ();
		else if (strcmp(keystring, "F7") == 0)
			stackCopyT();
		else if (strcmp(keystring, "F10") == 0)
			stackSign();
		else if (strcmp(keystring, "Prior") == 0)
			stackUp();
		else if (strcmp(keystring, "Next") == 0)
			stackDown();
		else {
			printf("%s (%02x) %s\n", str, str[0], keystring);
		}
		break;
	}
}

static long
BASE_ATOL(char*	xchar)
{
	long	l;

	switch (basesubmode) {
	case BASE_HEX:
		l = strtol(xchar, NULL, 16);
		break;
	case BASE_DEC:
		l = strtol(xchar, NULL, 10);
		break;
	case BASE_OCT:
		l = strtol(xchar, NULL, 8);
		break;
	case BASE_BIN:
		l = strtol(xchar, NULL, 2);
		break;
	default:
		l = strtol(xchar, NULL, 0);
		break;
	}

	return l;
}

static void
printstack_base(char *v, int index)
{
	unsigned int	i, flag;
	char	text[50];

	switch (basesubmode) {
	case BASE_HEX:
		sprintf(v, "%x", value_base[index]);
		break;
	case BASE_DEC:
		sprintf(v, "%d", value_base[index]);
		break;
	case BASE_OCT:
		sprintf(v, "%o", value_base[index]);
		break;
	case BASE_BIN:
		flag = 0;
		text[0] = '\0';
		for (i = 0x80000000; i; i >>= 1) {
			if (i & value_base[index]) {
				strcat(text, "1");
				flag = 1;
			} else {
				if (flag)
					strcat(text, "0");
			}
		}
		if (!flag)
			strcat(text, "0");
		strcpy(v, text);
		break;
	}
}

static void
setkeytop1_base(int code)
{
	XmString	str;

	str = XmStringCreateSimple(keytop[code]);
	XtVaSetValues(keyb[code], XmNlabelString, str, NULL);
	XmStringFree(str);
}

void
setkeytopall_base(void)
{
	int	i;

	for (i = 0; i < NKEY; i++)
		setkeytop1_base(i);
}

static void
clearstackall_base(void)
{
	int	i;

	for (i = 0; i < NSTACK; i++)
		value_base[i] = 0;
}

static void
stackupward_base(void)
{
	int	i;

	for (i = NSTACK - 1; i > 0; i--)
		value_base[i] = value_base[i - 1];
}

static void
stackEnter_base(char* xchar, int stackXpos)
{
	if (stackXpos > 0) {
		value_base[0] = BASE_ATOL(xchar);
	}

	stackupward_base();
}

static void
stackPush_base(void* d)
{
	stackupward_base();

	value_base[0] = *((int*)d);
}

static void*
stackPop_base(char* xchar, int stackXpos)
{
	static int	X;
	int	i;

	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);
	X = value_base[0];
	for (i = 0; i < NSTACK - 1; i++)
		value_base[i] = value_base[i + 1];

	return (void*)&X;
}

static void*
stackGetX_base(char *xchar, int stackXpos)
{
	static int	X;

	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	X = value_base[0];

	return (void*)&X;
}

static void
stackSetX_base(void* d)
{
	value_base[0] = *((int*)d);
}

static void
stackSwap_base(char* xchar, int stackXpos)
{
	int	X;

	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	X = value_base[0];
	value_base[0] = value_base[1];
	value_base[1] = X;
}

static void
stackUp_base(char* xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	stackupward_base();
}

static void
stackDown_base(void)
{
	int	i;

	for (i = 0; i < NSTACK - 1; i++)
		value_base[i] = value_base[i + 1];
}

static void
stackSign_base(char* xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	value_base[0] = -value_base[0];
}

static void
stackClearX_base(void)
{
	value_base[0] = 0.0;
}

static void
stackMem_base(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	memory_base = value_base[0];
}

static void
stackMemRead_base(void)
{
	value_base[0] = memory_base;
}

static void
stackMemAdd_base(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	memory_base += value_base[0];
}

static void
stackMemSub_base(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_base[0] = BASE_ATOL(xchar);

	memory_base -= value_base[0];
}

static void
stackCopyY_base(void)
{
	value_base[0] = value_base[1];
}

static void
stackCopyZ_base(void)
{
	value_base[0] = value_base[2];
}

static void
stackCopyT_base(void)
{
	value_base[0] = value_base[NSTACK - 1];
}

void
stack_basemode(struct stack_op_t **sp)
{
	*sp = &comp_stack_op;
}

void
keyboard_basemode(struct keyboard_op_t **kp)
{
	*kp = &comp_keyboard_op;
}

void
optionlabel_basemode(void)
{
	XmString	str;
	char	strchar[64];

	strcpy(strchar, "BASE");
	switch (basesubmode) {
	case BASE_HEX:
		strcat(strchar, " (HEX)");
		break;
	case BASE_DEC:
		strcat(strchar, " (DEC)");
		break;
	case BASE_OCT:
		strcat(strchar, " (OCT)");
		break;
	case BASE_BIN:
		strcat(strchar, " (BIN)");
		break;
	}

	str = XmStringCreateSimple(strchar);
	XtVaSetValues(optionlabel, XmNlabelString, str, NULL);
	XmStringFree(str);
}
