/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: compmode.c,v 1.9 1999/06/08 11:55:36 kato Exp $
 */

#include <Xm/Xm.h>

#include <stdlib.h>

#include "const.h"
#include "stack.h"
#include "keyboard.h"
#include "compmode.h"
#include "operator.h"
#include "option.h"

static void	clearstackall_comp(void);
static void	printstack_comp(char *v, int index);
static void	stackClearX_comp(void);
static void	stackCopyY_comp(void);
static void	stackCopyZ_comp(void);
static void	stackCopyT_comp(void);
static void	stackDown_comp(void);
static void	stackEnter_comp(char* xchar, int stackXpos);
static void*	stackGetX_comp(char *xchar, int stackXpos);
static void	stackMem_comp(char *xchar, int stackXpos);
static void	stackMemAdd_comp(char *xchar, int stackXpos);
static void	stackMemRead_comp(void);
static void	stackMemSub_comp(char *xchar, int stackXpos);
static void	stackPush_comp(void* d);
static void*	stackPop_comp(char* xchar, int stackXpos);
static void	stackSetX_comp(void* d);
static void	stackSign_comp(char* xchar, int stackXpos);
static void	stackSwap_comp(char* xchar, int stackXpos);
static void	stackUp_comp(char* xchar, int stackXpos);
static void	stackupward_comp(void);

static void	keyevent_comp(XKeyEvent* ev);
static void	keypush_comp(int keycode);
static void	setkeytop1_comp(int code);

static struct stack_op_t comp_stack_op = {
	printstack_comp,
	clearstackall_comp,
	stackupward_comp,
	stackEnter_comp,
	stackPush_comp,
	stackPop_comp,
	stackGetX_comp,
	stackSetX_comp,
	stackSwap_comp,
	stackUp_comp,
	stackDown_comp,
	stackSign_comp,
	stackClearX_comp,
	stackMem_comp,
	stackMemRead_comp,
	stackMemAdd_comp,
	stackMemSub_comp,
	stackCopyY_comp,
	stackCopyZ_comp,
	stackCopyT_comp,
};

static struct keyboard_op_t comp_keyboard_op = {
	setkeytop1_comp,
	keypush_comp,
	keyevent_comp,
	{
		"g", "h", "i", "j", "k", "l", "m",
		"G", "H", "I", "J", "K", "L", "M",
		"n", "o", "p", "q", "r", "s", "t",
		"ENTER",  "P", "Q", "R", "S", "T",
		"F1", "F2", "F3", "F4", "Tab", "Esc",
		"7", "8", "9", "BS", "Home", "Up",
		"4", "5", "6", "*", "/", "Down",
		"1", "2", "3", "+", "-", "F10",
		"0", ".", "e", "F5", "F6", "F7"
	}
};

static double	value_comp[NSTACK];
static double	memory_comp;

static char	*keytop[NKEY] = {
	"sin", "cos", "tan", "sinh", "cosh", "tanh", "PI",
	"asin", "acos", "atan", "asinh", "acosh", "atanh", "e",
	"1/X", "X!", "log", "ln", "sqrt", "Y^X", "Y<<X",
	"ENTER", "10^X", "e^X", "X^2", "cbrt", "Y>>X",
	"MEM", "MR", "M+", "M-", "SWAP", "OFF",
	"7", "8", "9", "BS", "CLR", "UP",
	"4", "5", "6", "*", "/", "DOWN",
	"1", "2", "3", "+", "-", "+/-",
	"0", ".", "EXP", "Y", "Z", "T"
};

static void
keypush_comp(int keycode)
{
	double	d;

	switch ((int)keycode) {
	case KEY_OFF:
		exit(0);
		/* NOTREACHED */
		break;
	case KEY_0:
		stackAddChar('0');
		break;
	case KEY_1:
		stackAddChar('1');
		break;
	case KEY_2:
		stackAddChar('2');
		break;
	case KEY_3:
		stackAddChar('3');
		break;
	case KEY_4:
		stackAddChar('4');
		break;
	case KEY_5:
		stackAddChar('5');
		break;
	case KEY_6:
		stackAddChar('6');
		break;
	case KEY_7:
		stackAddChar('7');
		break;
	case KEY_8:
		stackAddChar('8');
		break;
	case KEY_9:
		stackAddChar('9');
		break;
	case KEY_POINT:
		stackAddChar('.');
		break;
	case KEY_EXP:
		stackAddChar('e');
		break;
	case KEY_ADD:
		if (isjustaftere)
			stackAddChar('+');
		else {
			op_add_double();
		}
		break;
	case KEY_SUB:
		if (isjustaftere)
			stackAddChar('-');
		else {
			op_sub_double();
		}
		break;
	case KEY_MUL:
		op_mul_double();
		break;
	case KEY_DIV:
		op_div_double();
		break;
	case KEY_POW:
		op_pow_double();
		break;
	case KEY_SHIFTR:
		op_shiftR_double();
		break;
	case KEY_SHIFTL:
		op_shiftL_double();
		break;

	case KEY_SIN:
		op_sin();
		break;
	case KEY_COS:
		op_cos();
		break;
	case KEY_TAN:
		op_tan();
		break;
	case KEY_ASIN:
		op_asin();
		break;
	case KEY_ACOS:
		op_acos();
		break;
	case KEY_ATAN:
		op_atan();
		break;
	case KEY_SINH:
		op_sinh();
		break;
	case KEY_COSH:
		op_cosh();
		break;
	case KEY_TANH:
		op_tanh();
		break;
	case KEY_ASINH:
		op_asinh();
		break;
	case KEY_ACOSH:
		op_acosh();
		break;
	case KEY_ATANH:
		op_atanh();
		break;
	case KEY_LOG:
		op_log();
		break;
	case KEY_ANTILOG:
		op_antilog_double();
		break;
	case KEY_LN:
		op_ln();
		break;
	case KEY_ANTILN:
		op_antiln();
		break;
	case KEY_SQRT:
		op_sqrt_double();
		break;
	case KEY_CBRT:
		op_cbrt_double();
		break;
	case KEY_SQUARE:
		op_square_double();
		break;
	case KEY_RECIP:
		op_recip();
		break;
	case KEY_FACTO:
		op_facto_double();
		break;

	case KEY_SWAP:
		stackSwap();
		break;
	case KEY_BS:
		stackBS();
		break;
	case KEY_ENTER:
		stackEnter();
		break;
	case KEY_UP:
		stackUp();
		break;
	case KEY_DOWN:
		stackDown();
		break;
	case KEY_SIGN:
		stackSign();
		break;
	case KEY_CLR:
		stackClear();
		break;
	case KEY_MEM:
		stackMem();
		break;
	case KEY_MR:
		stackMemRead();
		break;
	case KEY_MPLUS:
		stackMemAdd();
		break;
	case KEY_MSUB:
		stackMemSub();
		break;
	case KEY_Y:
		stackCopyY();
		break;
	case KEY_Z:
		stackCopyZ();
		break;
	case KEY_T:
		stackCopyT();
		break;

	case KEY_PI:
		d = PI;
		stackPush(&d);
		break;
	case KEY_E:
		d = EXP;
		stackPush(&d);
		break;
	}
}

static void
keyevent_comp(XKeyEvent* ev)
{
	double	d;

	char	str[16], *keystring;
	KeySym	keysym;
	XComposeStatus	composestatus;
	int	len;

	len = XLookupString(ev, str, 15, &keysym, &composestatus);
	str[len] = '\0';

	keystring = XKeysymToString(keysym);

	switch (str[0]) {
	case 0x1b:	/* ESC */
		exit(0);
		/* NOTREACHED */
		break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case '.':
	case 'e':
		stackAddChar(str[0]);
		break;
	case '+':
		if (isjustaftere)
			stackAddChar(str[0]);
		else
			op_add_double();
		break;
	case '-':
		if (isjustaftere)
			stackAddChar(str[0]);
		else
			op_sub_double();
		break;
	case '*':
		op_mul_double();
		break;
	case '/':
		op_div_double();
		break;
	case '^':
		op_pow_double();
		break;

	case '!':
		op_facto_double();
		break;

	case 0x08:	/* BS */
		stackBS();
		break;
	case 0x0d:	/* Enter */
		stackEnter();
		break;

	case 'g':
		op_sin();
		break;
	case 'G':
		op_asin();
		break;
	case 'h':
		op_cos();
		break;
	case 'H':
		op_acos();
		break;
	case 'i':
		op_tan();
		break;
	case 'I':
		op_atan();
		break;
	case 'j':
		op_sinh();
		break;
	case 'J':
		op_asinh();
		break;
	case 'k':
		op_cosh();
		break;
	case 'K':
		op_acosh();
		break;
	case 'l':
		op_tanh();
		break;
	case 'L':
		op_atanh();
		break;
	case 'm':
		d = PI;
		stackPush(&d);
		break;
	case 'M':
		d = EXP;
		stackPush(&d);
		break;

	case 'n':
		op_recip();
		break;
	case 'o':
		op_facto_double();
		break;
	case 'p':
		op_log();
		break;
	case 'P':
		op_antilog_double();
		break;
	case 'q':
		op_ln();
		break;
	case 'Q':
		op_antiln();
		break;
	case 'r':
		op_sqrt_double();
		break;
	case 'R':
		op_square_double();
		break;
	case 's':
		op_pow_double();
		break;
	case 'S':
		op_cbrt_double();
		break;
	case 't':
		op_shiftL_double();
		break;
	case 'T':
		op_shiftR_double();
		break;

	case 9:	/* TAB */
		stackSwap();
		break;

	default:
		if (strcmp(keystring, "Home") == 0)
			stackClear();
		else if (strcmp(keystring, "F1") == 0)
			stackMem();
		else if (strcmp(keystring, "F2") == 0)
			stackMemRead();
		else if (strcmp(keystring, "F3") == 0)
			stackMemAdd();
		else if (strcmp(keystring, "F4") == 0)
			stackMemSub();
		else if (strcmp(keystring, "F5") == 0)
			stackCopyY();
		else if (strcmp(keystring, "F6") == 0)
			stackCopyZ();
		else if (strcmp(keystring, "F7") == 0)
			stackCopyT();
		else if (strcmp(keystring, "F10") == 0)
			stackSign();
		else if (strcmp(keystring, "Prior") == 0)
			stackUp();
		else if (strcmp(keystring, "Next") == 0)
			stackDown();
		else {
			printf("%s (%02x) %s\n", str, str[0], keystring);
		}
		break;
	}
}

static void
printstack_comp(char *v, int index)
{
	sprintf(v, "%.10g", value_comp[index]);
}

static void
setkeytop1_comp(int code)
{
	XmString	str;

	str = XmStringCreateSimple(keytop[code]);
	XtVaSetValues(keyb[code], XmNlabelString, str, NULL);
	XmStringFree(str);
}

void
setkeytopall_comp(void)
{
	int	i;

	for (i = 0; i < NKEY; i++)
		setkeytop1_comp(i);
}

static void
clearstackall_comp(void)
{
	int	i;

	for (i = 0; i < NSTACK; i++)
		value_comp[i] = 0.0;
}

static void
stackupward_comp(void)
{
	int	i;

	for (i = NSTACK - 1; i > 0; i--)
		value_comp[i] = value_comp[i - 1];
}

static void
stackEnter_comp(char* xchar, int stackXpos)
{
	if (stackXpos > 0) {
		value_comp[0] = atof(xchar);
	}

	stackupward_comp();
}

static void
stackPush_comp(void* d)
{
	stackupward_comp();

	value_comp[0] = *((double*)d);
}

static void*
stackPop_comp(char* xchar, int stackXpos)
{
	static double	X;
	int	i;

	if (stackXpos > 0)
		value_comp[0] = atof(xchar);
	X = value_comp[0];
	for (i = 0; i < NSTACK - 1; i++)
		value_comp[i] = value_comp[i + 1];

	return (void*)&X;
}

static void*
stackGetX_comp(char *xchar, int stackXpos)
{
	static double	X;

	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	X = value_comp[0];

	return (void*)&X;
}

static void
stackSetX_comp(void* d)
{
	value_comp[0] = *((double*)d);
}

static void
stackSwap_comp(char* xchar, int stackXpos)
{
	double	X;

	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	X = value_comp[0];
	value_comp[0] = value_comp[1];
	value_comp[1] = X;
}

static void
stackUp_comp(char* xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	stackupward_comp();
}

static void
stackDown_comp(void)
{
	int	i;

	for (i = 0; i < NSTACK - 1; i++)
		value_comp[i] = value_comp[i + 1];
}

static void
stackSign_comp(char* xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	value_comp[0] = -value_comp[0];
}

static void
stackClearX_comp(void)
{
	value_comp[0] = 0.0;
}

static void
stackMem_comp(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	memory_comp = value_comp[0];
}

static void
stackMemRead_comp(void)
{
	value_comp[0] = memory_comp;
}

static void
stackMemAdd_comp(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	memory_comp += value_comp[0];
}

static void
stackMemSub_comp(char *xchar, int stackXpos)
{
	if (stackXpos > 0)
		value_comp[0] = atof(xchar);

	memory_comp -= value_comp[0];
}

static void
stackCopyY_comp(void)
{
	value_comp[0] = value_comp[1];
}

static void
stackCopyZ_comp(void)
{
	value_comp[0] = value_comp[2];
}

static void
stackCopyT_comp(void)
{
	value_comp[0] = value_comp[NSTACK - 1];
}

void
stack_compmode(struct stack_op_t **sp)
{
	*sp = &comp_stack_op;
}

void
keyboard_compmode(struct keyboard_op_t **kp)
{
	*kp = &comp_keyboard_op;
}

void
optionlabel_compmode(void)
{
	XmString	str;

	str = XmStringCreateSimple("COMP");
	XtVaSetValues(optionlabel, XmNlabelString, str, NULL);
	XmStringFree(str);
}
