/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: keyboard.c,v 1.15 1999/06/08 10:02:38 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/Form.h>
#include <Xm/PushB.h>

#include <stdio.h>
#include <stdlib.h>

#include "stack.h"
#include "keyboard.h"
#include "const.h"
#include "compmode.h"
#include "operator.h"
#include "option.h"

struct keyposition keypos[NKEY] = {
	{ 2, 14,  2, 11, KEY_SIN},
	{16, 28,  2, 11, KEY_COS},
	{30, 42,  2, 11, KEY_TAN},
	{44, 56,  2, 11, KEY_SINH},
	{58, 70,  2, 11, KEY_COSH},
	{72, 84,  2, 11, KEY_TANH},
	{86, 98,  2, 11, KEY_PI},

	{ 2, 14, 13, 22, KEY_ASIN},
	{16, 28, 13, 22, KEY_ACOS},
	{30, 42, 13, 22, KEY_ATAN},
	{44, 56, 13, 22, KEY_ASINH},
	{58, 70, 13, 22, KEY_ACOSH},
	{72, 84, 13, 22, KEY_ATANH},
	{86, 98, 13, 22, KEY_E},

	{ 2, 14, 24, 33, KEY_RECIP},
	{16, 28, 24, 33, KEY_FACTO},
	{30, 42, 24, 33, KEY_LOG},
	{44, 56, 24, 33, KEY_LN},
	{58, 70, 24, 33, KEY_SQRT},
	{72, 84, 24, 33, KEY_POW},
	{86, 98, 24, 33, KEY_SHIFTL},

	{ 2, 28, 35, 44, KEY_ENTER},
	{30, 42, 35, 44, KEY_ANTILOG},
	{44, 56, 35, 44, KEY_ANTILN},
	{58, 70, 35, 44, KEY_SQUARE},
	{72, 84, 35, 44, KEY_CBRT},
	{86, 98, 35, 44, KEY_SHIFTR},


	{ 2, 16, 46, 55, KEY_MEM},
	{19, 33, 46, 55, KEY_MR},
	{35, 49, 46, 55, KEY_MPLUS},
	{51, 66, 46, 55, KEY_MSUB},
	{68, 82, 46, 55, KEY_SWAP},
	{85, 99, 46, 55, KEY_OFF},

	{ 2, 16, 57, 66, KEY_7},
	{19, 33, 57, 66, KEY_8},
	{35, 49, 57, 66, KEY_9},
	{51, 66, 57, 66, KEY_BS},
	{68, 82, 57, 66, KEY_CLR},
	{85, 99, 57, 66, KEY_UP},

	{ 2, 16, 68, 77, KEY_4},
	{19, 33, 68, 77, KEY_5},
	{35, 49, 68, 77, KEY_6},
	{51, 66, 68, 77, KEY_MUL},
	{68, 82, 68, 77, KEY_DIV},
	{85, 99, 68, 77, KEY_DOWN},

	{ 2, 16, 79, 88, KEY_1},
	{19, 33, 79, 88, KEY_2},
	{35, 49, 79, 88, KEY_3},
	{51, 66, 79, 88, KEY_ADD},
	{68, 82, 79, 88, KEY_SUB},
	{85, 99, 79, 88, KEY_SIGN},

	{ 2, 16, 90, 99, KEY_0},
	{19, 33, 90, 99, KEY_POINT},
	{35, 49, 90, 99, KEY_EXP},
	{51, 66, 90, 99, KEY_Y},
	{68, 82, 90, 99, KEY_Z},
	{85, 99, 90, 99, KEY_T},
};

Widget	keyb[NKEY];

/* ARGSUSED */
static void
pushbCB(Widget w, XtPointer keycode, XtPointer dum)
{
	keyboard_op->keypush((int)keycode);
}

static void
createkey(Widget parent)
{
	char	s[8];
	int	i;

	for (i = 0; i < NKEY; i++) {
		sprintf(s, "key%d", i);
		keyb[i] = XtVaCreateWidget(s,
		    xmPushButtonWidgetClass, parent,
		    XmNtopAttachment, XmATTACH_POSITION,
		    XmNtopPosition, keypos[i].y1,
		    XmNleftAttachment, XmATTACH_POSITION,
		    XmNleftPosition, keypos[i].x1,
		    XmNrightAttachment, XmATTACH_POSITION,
		    XmNrightPosition, keypos[i].x2,
		    XmNbottomAttachment, XmATTACH_POSITION,
		    XmNbottomPosition, keypos[i].y2,
		    NULL);
		keyboard_op->setkeytop1(i);
		XtManageChild(keyb[i]);
		XtAddCallback(keyb[i], XmNactivateCallback,
		    (XtCallbackProc)pushbCB,
		    (XtPointer)keypos[i].keycode);
	}
}

Widget
initkeyboard(Widget parent, Widget stack)
{
	Widget keyform;

	setkeyboardop();

	keyform = XtVaCreateWidget("keyform",
	    xmFormWidgetClass, parent,
	    XmNtopAttachment, XmATTACH_WIDGET,
	    XmNtopWidget, stack,
	    XmNleftAttachment, XmATTACH_FORM,
	    XmNrightAttachment, XmATTACH_FORM,
	    XmNbottomAttachment, XmATTACH_FORM,
	    NULL);
	createkey(keyform);
	XtManageChild(keyform);

	return keyform;
}

void
keyevent(XKeyEvent* ev)
{
	keyboard_op->keyevent(ev);
}
