/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: main.c,v 1.8 1999/06/09 06:44:45 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/MainW.h>

#include <signal.h>

#include "keyboard.h"
#include "console.h"
#include "menubar.h"
#include "help.h"

static String	fallback_resources[] = {
	"*albl1.fontList:	-*-lucida-bold-r-*-*-18-*-*-*-*-*-iso8859-1=LARGE,"
	"		-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1=NORMAL,",
	"*a_bt.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",

	"*label00.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*label01.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*label02.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*label03.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*label04.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*labelT.fontList:	-*-lucida-bold-r-*-*-14-*-*-*-*-*-iso8859-1",
	"*stack00.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
	"*stack01.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
	"*stack02.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
	"*stack03.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
	"*stack04.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",
	"*stackT.fontList:	-*-lucida-medium-r-*-*-12-*-*-*-*-*-iso8859-1",

	"*degtitle.labelString: Angle",
	"*degtitle.fontList:	-*-times-medium-r-*-*-10-*-*-*-*-*-iso8859-1",
	"*degree.labelString: DEG",
	"*degree.fontList:	-*-times-medium-r-*-*-10-*-*-*-*-*-iso8859-1",
	"*radian.labelString: RAD",
	"*radian.fontList:	-*-times-medium-r-*-*-10-*-*-*-*-*-iso8859-1",
	"*gradian.labelString: GRAD",
	"*gradian.fontList:	-*-times-medium-r-*-*-10-*-*-*-*-*-iso8859-1",

	"*optionmenu.labelString: Option",
	"*opttionmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*anglemenu.labelString: Angle",
	"*anglemenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*degmenu.labelString: Degree",
	"*degmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*radmenu.labelString: Radian",
	"*radmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*gradmenu.labelString: Gradian",
	"*gradmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*modemenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*modemenu.labelString: Mode",
	"*compmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*compmenu.labelString: Comp",
	"*basemenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*basemenu.labelString: Base",

	"*filemenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*filemenu.labelString:	File",
	"*filemenu.mnemonic:	F",
	"*exit.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*exit.labelString:	Exit",
	"*exit.mnemonic:	x",

	"*helpmenu.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*helpmenu.labelString:	Help",
	"*filemenu.mnemonic:	H",
	"*about.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*about.labelString:	About...",
	"*about.mnemonic:	A",
	"*shortcut.fontList:	-*-helvetica-medium-r-*-*-*-120-*--*-*-iso8859-*",
	"*shortcut.labelString:	Shortcut",
	"*shortcut.mnemonic:	S",

	"*sckey.labelString:	Q",
	NULL
};

int
main(int argc, char** argv)
{
	XEvent	ev;
	XtAppContext	app_context;
	Widget	toplevel;
	Widget	mainwindow;
	Widget	menubar;
	Widget	workarea;
	Arg	args[20];
	Cardinal	n;

	n = 0;
	XtSetArg(args[n], XmNallowResize, True); n++;

	toplevel = XtAppInitialize(&app_context,
	    "Mrpnc",
	    (XrmOptionDescList)NULL,
	    0, 
	    (Cardinal*)&argc,
	    argv,
	    fallback_resources,
	    args, n);

	n = 0;
	XtSetArg(args[n],
	    XmNcommandWindowLocation, XmCOMMAND_ABOVE_WORKSPACE); n++;
	mainwindow = XmCreateMainWindow(toplevel, "mainwin", args, n);

	workarea = initconsole(mainwindow);
	menubar = initmenu(mainwindow);
	inithelp(mainwindow);

	XmMainWindowSetAreas(mainwindow,
	    menubar,
	    (Widget)NULL,
	    (Widget)NULL,
	    (Widget)NULL,
	    (Widget)workarea);
	XtManageChild(mainwindow);

	XtRealizeWidget(toplevel);

	signal(SIGFPE, SIG_IGN);

	for (;;) {
		XtAppNextEvent(app_context, &ev);
		switch (ev.xany.type) {
		case KeyPress:
			keyevent((XKeyEvent*)&ev);
			break;
		default:
			XtDispatchEvent(&ev);
		}
	}

	return 0;
}
