/*
 * Copyright (c) KATO Takenori, 1999.
 * 
 * All rights reserved.  Unpublished rights reserved under the copyright
 * laws of Japan.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * 	$Id: menubar.c,v 1.4 1999/06/08 11:55:36 kato Exp $
 */

#include <Xm/Xm.h>
#include <Xm/CascadeB.h>
#include <Xm/PushB.h>
#include <Xm/RowColumn.h>

#include <stdlib.h>

#include "keyboard.h"
#include "stack.h"
#include "help.h"
#include "option.h"
#include "menubar.h"

#define	FILE_EXIT	99
#define	HELP_SHORTCUT	1
#define	HELP_ABOUT	99

static void
AngleMenuCB(Widget w, XtPointer clientdata, XtPointer calldata)
{
	setangle((int)clientdata);
}

static void
ModeMenuCB(Widget w, XtPointer clientdata, XtPointer calldata)
{
	setmode((int)clientdata);
}

static void
FileMenuCB(Widget w, XtPointer clientdata, XtPointer calldata)
{
	switch ((int)clientdata) {
	case FILE_EXIT:
		exit(0);
		/* NOTREACHED */
		break;
	}
}

static void
HelpMenuCB(Widget w, XtPointer clientdata, XtPointer calldata)
{
	switch ((int)clientdata) {
	case HELP_ABOUT:
		DisplayAboutDialog();
		break;
	case HELP_SHORTCUT:
		DisplayShortcutDialog();
		break;
	}
}

static Widget
CreateButton(Widget parent, char* name,
    XtCallbackProc callback, XtPointer clientdata)
{
	Widget	button;

	button = XtVaCreateWidget(name,
	    xmPushButtonWidgetClass, parent,
	    NULL);
	XtAddCallback(button, XmNactivateCallback, callback, clientdata);
	XtManageChild(button);

	return button;
}

static Widget
CreatePulldownMenu(Widget parent, char* name)
{
	Widget	menu;
	Widget	cascade;
	Arg	args[2];
	Cardinal	n;

	n = 0;
	menu = XmCreatePulldownMenu(parent, name, args, n);

	cascade = XtVaCreateManagedWidget(name,
	    xmCascadeButtonWidgetClass, parent,
	    XmNsubMenuId, menu,
	    NULL);

	return menu;
}

static Widget
CreateHelp(Widget parent)
{
	Widget	menu, cascade;
	Arg	args[2];
	Cardinal	n;

	n = 0;
	menu = XmCreatePulldownMenu(parent, "helpmenu", args, n);

	cascade = XtVaCreateManagedWidget("helpmenu",
	    xmCascadeButtonWidgetClass, parent,
	    XmNsubMenuId, menu,
	    NULL);

	XtVaSetValues(parent, XmNmenuHelpWidget, cascade, NULL);

	return menu;
}

Widget
initmenu(Widget parent)
{
	Widget	menubar, filemenu, helpmenu;
	Widget	optionmenu, anglemenu, modemenu;

	Arg	args[2];
	Cardinal	n;

	n = 0;
	menubar = XmCreateMenuBar(parent, "menubar", args, n);
	XtManageChild(menubar);

	filemenu = CreatePulldownMenu(menubar, "filemenu");
	(void)CreateButton(filemenu, "exit", (XtCallbackProc)FileMenuCB,
	    (XtPointer)FILE_EXIT);

	helpmenu = CreateHelp(menubar);
	(void)CreateButton(helpmenu, "shortcut", (XtCallbackProc)HelpMenuCB,
	    (XtPointer)HELP_SHORTCUT);
	(void)CreateButton(helpmenu, "about", (XtCallbackProc)HelpMenuCB,
	    (XtPointer)HELP_ABOUT);

	optionmenu = CreatePulldownMenu(menubar, "optionmenu");
	anglemenu = CreatePulldownMenu(optionmenu, "anglemenu");
	(void)CreateButton(anglemenu, "degmenu", (XtCallbackProc)AngleMenuCB,
	    (XtPointer)ANGLE_DEG);
	(void)CreateButton(anglemenu, "radmenu", (XtCallbackProc)AngleMenuCB,
	    (XtPointer)ANGLE_RAD);
	(void)CreateButton(anglemenu, "gradmenu", (XtCallbackProc)AngleMenuCB,
	    (XtPointer)ANGLE_GRAD);
	modemenu = CreatePulldownMenu(optionmenu, "modemenu");
	(void)CreateButton(modemenu, "compmenu", (XtCallbackProc)ModeMenuCB,
	    (XtPointer)MODE_COMP);
	(void)CreateButton(modemenu, "basemenu", (XtCallbackProc)ModeMenuCB,
	    (XtPointer)MODE_BASE);

	return menubar;
}
